' This file is part of Network Switcher.

' Network Switcher is free software: you can redistribute it and/or modify
' it under the terms of the GNU General Public License as published by
' the Free Software Foundation, either version 3 of the License, or
' (at your option) any later version.

' Network Switcher is distributed in the hope that it will be useful,
' but WITHOUT ANY WARRANTY; without even the implied warranty of
' MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
' GNU General Public License for more details.

' You should have received a copy of the GNU General Public License
' along with Network Switcher. If not, see <http://www.gnu.org/licenses/>.
'
' Copyright 2007, Ricardo Amaral

Imports NetworkSwitcher.Interfaces

Public Class FormNewProfile

#Region " Module Variables "

    Dim LastSelectedItem(1) As Object

#End Region

#Region " Private Methods "

    ' Updates combo boxes with available and selectable NICs
    Private Sub UpdateComboBox(ByVal CB1 As ComboBox, ByVal CB2 As ComboBox, ByVal X As Integer)
        For I As Integer = 0 To CB2.Items.Count - 1
            If CB2.Items.Item(I).Equals(CB1.SelectedItem) Then
                CB2.Items.Remove(CB1.SelectedItem)
                Exit For
            End If
        Next

        If LastSelectedItem(X) IsNot Nothing Then
            CB2.Items.Add(LastSelectedItem(X))
        End If

        LastSelectedItem(X) = CB1.SelectedItem
    End Sub

    ' Enables/Disables DHCP related fields
    Private Sub ChangeDHCPFields(ByVal conn As Integer)
        Select Case conn
            Case 1
                If chkDHCPN1.Checked = True Then
                    ' Disable labels
                    Me.lblIPN1.Enabled = False
                    Me.lblSubnetN1.Enabled = False
                    Me.lblGatewayN1.Enabled = False
                    Me.lblDNS1N1.Enabled = False
                    Me.lblDNS2N1.Enabled = False

                    ' Disable textboxes
                    Me.txtIPN1.Enabled = False
                    Me.txtSubnetN1.Enabled = False
                    Me.txtGatewayN1.Enabled = False
                    Me.txtDNS1N1.Enabled = False
                    Me.txtDNS2N1.Enabled = False
                Else
                    ' Enable labels
                    Me.lblIPN1.Enabled = True
                    Me.lblSubnetN1.Enabled = True
                    Me.lblGatewayN1.Enabled = True
                    Me.lblDNS1N1.Enabled = True
                    Me.lblDNS2N1.Enabled = True

                    ' Enable textboxes
                    Me.txtIPN1.Enabled = True
                    Me.txtSubnetN1.Enabled = True
                    Me.txtGatewayN1.Enabled = True
                    Me.txtDNS1N1.Enabled = True
                    Me.txtDNS2N1.Enabled = True
                End If
            Case 2
                If chkDHCPN2.Checked = True Then
                    ' Disable labels
                    Me.lblIPN2.Enabled = False
                    Me.lblSubnetN2.Enabled = False
                    Me.lblWGateway.Enabled = False
                    Me.lblDNS1N2.Enabled = False
                    Me.lblDNS2N2.Enabled = False

                    ' Disable textboxes
                    Me.txtIPN2.Enabled = False
                    Me.txtSubnetN2.Enabled = False
                    Me.txtGatewayN2.Enabled = False
                    Me.txtDNS1N2.Enabled = False
                    Me.txtDNS2N2.Enabled = False
                Else
                    ' Enable labels
                    Me.lblIPN2.Enabled = True
                    Me.lblSubnetN2.Enabled = True
                    Me.lblWGateway.Enabled = True
                    Me.lblDNS1N2.Enabled = True
                    Me.lblDNS2N2.Enabled = True

                    ' Enable textboxes
                    Me.txtIPN2.Enabled = True
                    Me.txtSubnetN2.Enabled = True
                    Me.txtGatewayN2.Enabled = True
                    Me.txtDNS1N2.Enabled = True
                    Me.txtDNS2N2.Enabled = True
                End If
        End Select
    End Sub

    ' Save the new profile
    Private Sub SaveNewProfile()
        Dim Gateway As String = Nothing
        Dim DNS1 As String = Nothing
        Dim DNS2 As String = Nothing
        Dim Aux As String

        ' Initialize XML profiles file
        XML = New XMLParser(Application.StartupPath, "profiles", "Profiles")

        ' Save NIC1 settings to XML file
        If Me.cbxNIC1.SelectedItem IsNot Nothing Then
            XML.SaveSettingStr(profileName, "NIC1Enable", Me.chkEnableN1.Checked)
            XML.SaveSettingStr(profileName, "NIC1Name", Me.cbxNIC1.SelectedItem.ToString)

            If Not Me.chkEnableN1.Checked Then
                Me.chkDHCPN1.Checked = False
            End If

            XML.SaveSettingStr(profileName, "NIC1DHCP", Me.chkDHCPN1.Checked.ToString)

            ' Checks if the Gateway, DNS1 and DNS2 are not "null"
            If Me.txtGatewayN1.ToString = "0.0.0.0" Then Gateway = Nothing Else Gateway = Me.txtGatewayN1.ToString
            If Me.txtDNS1N1.ToString = "0.0.0.0" Then DNS1 = Nothing Else DNS1 = Me.txtDNS1N1.ToString
            If Me.txtDNS2N1.ToString = "0.0.0.0" Then DNS2 = Nothing Else DNS2 = Me.txtDNS2N1.ToString

            ' Swap DNSs if only DNS2 was filled
            If DNS1 = Nothing And Not DNS2 = Nothing Then
                Aux = DNS1
                DNS1 = DNS2
                DNS2 = Aux
            End If

            ' Are we using DHCP for network connection 1?
            If (Me.chkEnableN1.Checked And Not Me.chkDHCPN1.Checked) And Not Me.chkDHCPN1.Checked Then
                XML.SaveSettingStr(profileName, "NIC1IP", Me.txtIPN1.ToString)
                XML.SaveSettingStr(profileName, "NIC1Subnet", Me.txtSubnetN1.ToString)
                XML.SaveSettingStr(profileName, "NIC1Gateway", Gateway)
                XML.SaveSettingStr(profileName, "NIC1DNS1", DNS1)
                XML.SaveSettingStr(profileName, "NIC1DNS2", DNS2)
            Else
                XML.SaveSettingStr(profileName, "NIC1IP", Nothing)
                XML.SaveSettingStr(profileName, "NIC1Subnet", Nothing)
                XML.SaveSettingStr(profileName, "NIC1Gateway", Nothing)
                XML.SaveSettingStr(profileName, "NIC1DNS1", Nothing)
                XML.SaveSettingStr(profileName, "NIC1DNS2", Nothing)
            End If
        Else
            XML.SaveSettingStr(profileName, "NIC1Enable", "False")
            XML.SaveSettingStr(profileName, "NIC1Name", Nothing)
            XML.SaveSettingStr(profileName, "NIC1DHCP", "False")
            XML.SaveSettingStr(profileName, "NIC1IP", Nothing)
            XML.SaveSettingStr(profileName, "NIC1Subnet", Nothing)
            XML.SaveSettingStr(profileName, "NIC1Gateway", Nothing)
            XML.SaveSettingStr(profileName, "NIC1DNS1", Nothing)
            XML.SaveSettingStr(profileName, "NIC1DNS2", Nothing)
        End If

        ' Save NIC2 settings to XML file
        If Me.cbxNIC2.SelectedItem IsNot Nothing Then
            XML.SaveSettingStr(profileName, "NIC2Enable", Me.chkEnableN2.Checked)
            XML.SaveSettingStr(profileName, "NIC2Name", Me.cbxNIC2.SelectedItem.ToString)

            If Not Me.chkEnableN2.Checked Then
                Me.chkDHCPN2.Checked = False

            End If
            XML.SaveSettingStr(profileName, "NIC2DHCP", Me.chkDHCPN2.Checked.ToString)

            ' Checks if the Gateway, DNS1 and DNS2 are not "null"
            If Me.txtGatewayN2.ToString = "0.0.0.0" Then Gateway = Nothing Else Gateway = Me.txtGatewayN2.ToString
            If Me.txtDNS1N2.ToString = "0.0.0.0" Then DNS1 = Nothing Else DNS1 = Me.txtDNS1N2.ToString
            If Me.txtDNS2N2.ToString = "0.0.0.0" Then DNS2 = Nothing Else DNS2 = Me.txtDNS2N2.ToString

            ' Swap DNSs if only DNS2 was filled
            If DNS1 = Nothing And Not DNS2 = Nothing Then
                Aux = DNS1
                DNS1 = DNS2
                DNS2 = Aux
            End If

            ' Are we using DHCP for network connection 2?
            If (Me.chkEnableN2.Checked And Not Me.chkDHCPN2.Checked) And Not Me.chkDHCPN2.Checked Then
                XML.SaveSettingStr(profileName, "NIC2IP", Me.txtIPN2.ToString)
                XML.SaveSettingStr(profileName, "NIC2Subnet", Me.txtSubnetN2.ToString)
                XML.SaveSettingStr(profileName, "NIC2Gateway", Gateway)
                XML.SaveSettingStr(profileName, "NIC2DNS1", DNS1)
                XML.SaveSettingStr(profileName, "NIC2DNS2", DNS2)
            Else
                XML.SaveSettingStr(profileName, "NIC2IP", Nothing)
                XML.SaveSettingStr(profileName, "NIC2Subnet", Nothing)
                XML.SaveSettingStr(profileName, "NIC2Gateway", Nothing)
                XML.SaveSettingStr(profileName, "NIC2DNS1", Nothing)
                XML.SaveSettingStr(profileName, "NIC2DNS2", Nothing)
            End If
        Else
            XML.SaveSettingStr(profileName, "NIC2Enable", "False")
            XML.SaveSettingStr(profileName, "NIC2Name", Nothing)
            XML.SaveSettingStr(profileName, "NIC2DHCP", "False")
            XML.SaveSettingStr(profileName, "NIC2IP", Nothing)
            XML.SaveSettingStr(profileName, "NIC2Subnet", Nothing)
            XML.SaveSettingStr(profileName, "NIC2Gateway", Nothing)
            XML.SaveSettingStr(profileName, "NIC2DNS1", Nothing)
            XML.SaveSettingStr(profileName, "NIC2DNS2", Nothing)
        End If

        ' Save selected plugins and their selected methods to XML file
        If Me.clbPluginsList.CheckedItems.Count > 0 Then
            Dim pID As Integer = 0
            Dim mNames As String = Nothing

            ' Save selected plugins count
            XML.SaveSettingInt(profileName, "PluginsCount", Me.clbPluginsList.CheckedItems.Count)

            ' Loop through all plugins in the available plugins structure
            For Each pmItem As StructAvailablePlugins In Plugins
                ' Loop through all checked methods in this plugin
                For I As Integer = 0 To pmItem.Checked.Count - 1
                    ' Save this method if it's checked
                    If pmItem.Checked(I) = CheckState.Checked Then
                        If mNames = Nothing Then
                            mNames = pmItem.Methods(I)
                        Else
                            mNames = mNames + ";" + pmItem.Methods(I)
                        End If
                    End If
                Next

                ' Do we have any plugin methods to save?
                If Not mNames = Nothing Then
                    XML.SaveSettingStr(profileName, "PluginName(" + CStr(pID) + ")", pmItem.Name)
                    XML.SaveSettingStr(profileName, "PluginMethods(" + CStr(pID) + ")", mNames)

                    ' Increments plugin ID and reset method names
                    pID = pID + 1
                    mNames = Nothing
                End If
            Next
        End If

        ' Is this a new profile or are we editing?
        If newProfile And Not editProfile Then
            ' Add new profile to ArrayList and sort it
            listProfiles.Add(profileName)
            listProfiles.Sort()

            ' Re-populates profiles combobox and notify menu
            RefreshProfiles(profileName)
        End If
    End Sub

    ' Method: Gets all plugins and their methods names assigned to a profile
    Private Function SetPluginsMethodsCheckState(ByVal profile As String) As ArrayList
        Dim pCount As Integer = XML.GetSettingInt(profile, "PluginsCount", 0)
        Dim apName As String, apMethods As String
        Dim pNames As New ArrayList

        ' Loops through all plugins assigned to this profile
        For I As Integer = 0 To pCount - 1
            ' Get the plugin name and methods from the XML
            apName = XML.GetSettingStr(profile, "PluginName(" + CStr(I) + ")", Nothing)
            apMethods = XML.GetSettingStr(profile, "PluginMethods(" + CStr(I) + ")", Nothing)

            ' Loops through all avaialble plugins in the structure
            For Each pmItem As StructAvailablePlugins In Plugins
                If apName = pmItem.Name Then
                    For K As Integer = 0 To pmItem.Methods.Count - 1
                        ' Loops through all methods assigned to this plugin and check their states
                        For Each method As String In apMethods.Split(";")
                            ' Check it if we found a matching method
                            If pmItem.Methods(K) = method Then
                                pmItem.Checked(K) = CheckState.Checked

                                Exit For
                            End If
                        Next
                    Next
                End If
            Next

            ' Adds this assigned plugin name to a ArrayList
            pNames.Add(apName)
        Next

        Return pNames
    End Function

#End Region

#Region " Controls Events "

    ' DHCP checkboxes related events
    Private Sub chkLDHCP_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles chkDHCPN1.CheckedChanged
        ChangeDHCPFields(1)
    End Sub

    Private Sub chkWDHCP_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles chkDHCPN2.CheckedChanged
        ChangeDHCPFields(2)
    End Sub

    ' Connections checkboxes related events
    Private Sub chkLEnable_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles chkEnableN1.CheckedChanged
        If Me.chkEnableN1.Checked = True Then
            Me.gbxEnableN1.Enabled = True
        Else
            Me.gbxEnableN1.Enabled = False
        End If
    End Sub

    Private Sub chkWEnable_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles chkEnableN2.CheckedChanged
        If Me.chkEnableN2.Checked = True Then
            Me.gbxEnableN2.Enabled = True
        Else
            Me.gbxEnableN2.Enabled = False
        End If
    End Sub

    ' Save profile button event
    Private Sub btnSaveProfle_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnSaveProfle.Click
        ' If everything's OK, save the new profile
        If saveProfile Then
            ' Throw error if something is wrong
            If Me.cbxNIC1.SelectedItem = Nothing And Me.cbxNIC2.SelectedItem = Nothing Then
                MessageBox.Show("The profile can't be saved, at least one connection must be configured.", "Can't save new profile!", MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
            ElseIf (Me.chkEnableN1.Checked And Me.cbxNIC1.SelectedItem = Nothing) Or _
                   (Me.chkEnableN2.Checked And Me.cbxNIC2.SelectedItem = Nothing) Then
                MessageBox.Show("The profile can't be saved, there is no name for the enabled connection(s).", "Can't save new profile!", MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
            ElseIf (Me.chkEnableN1.Checked And Not Me.chkDHCPN1.Checked And _
                   (Me.txtIPN1.ToString = "0.0.0.0" Or Me.txtSubnetN1.ToString = "0.0.0.0")) Or _
                   (Me.chkEnableN2.Checked And Not Me.chkDHCPN2.Checked And _
                   (Me.txtIPN2.ToString = "0.0.0.0" Or Me.txtSubnetN2.ToString = "0.0.0.0")) Then
                MessageBox.Show("The profile can't be saved, IP Address and/or Subnet Mask fields not filled.", "Can't save new profile!", MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
            Else
                ' Saves the new profile
                SaveNewProfile()

                ' Closes this form instance
                Me.Close()
            End If
        Else
            MessageBox.Show("The profile can't be saved, check all the red exclamation points.", "Can't save new profile!", MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
        End If
    End Sub

    ' Lists all the actions (methods) for the selected plugin
    Private Sub clbPluginsList_SelectedIndexChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles clbPluginsList.SelectedIndexChanged
        Dim I As Integer

        ' Sets the plugin actions groupbox enabled state
        With Me.clbPluginsList
            If .GetItemCheckState(.SelectedIndex) = CheckState.Checked Then
                Me.gbxPluginActions.Enabled = True
            ElseIf .GetItemCheckState(.SelectedIndex) = CheckState.Unchecked Then
                Me.gbxPluginActions.Enabled = False
            End If
        End With

        ' Loops through all loaded plugins
        For Each pmItem As StructAvailablePlugins In Plugins
            ' Is this the selected plugin?
            If pmItem.Name = Me.clbPluginsList.SelectedItem Then
                Me.clbPluginActions.ClearSelected()
                Me.clbPluginActions.Items.Clear()

                ' Loops through all methods in the available plugins structure
                For Each method As String In pmItem.Methods
                    ' Add this method (checked or unchecked) to the plugin actions
                    If pmItem.Checked(I) = CheckState.Checked Then
                        Me.clbPluginActions.Items.Add(method, True)
                    Else
                        Me.clbPluginActions.Items.Add(method, False)
                    End If

                    I = I + 1
                Next
                Exit For
            End If
        Next
    End Sub

    ' Changes the check state in the ArrayList for the selected plugin action
    Private Sub clbPluginActions_ItemCheck(ByVal sender As Object, ByVal e As System.Windows.Forms.ItemCheckEventArgs) Handles clbPluginActions.ItemCheck
        Dim I As Integer

        ' Loops through all plugins in the available plugins structure
        For Each plugin As StructAvailablePlugins In Plugins
            ' Is this the selected plugin?
            If plugin.Name = Me.clbPluginsList.SelectedItem Then
                ' Loops through all methods in this plugin
                For Each method As String In plugin.Methods
                    ' If this is our method, set new checked state
                    If method = Me.clbPluginActions.SelectedItem Then
                        plugin.Checked(I) = e.NewValue

                        Exit For
                    End If

                    I = I + 1
                Next
                Exit For
            End If
        Next
    End Sub

    ' Moves the selected plugin action item one position up
    Private Sub btnMoveActionUp_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnMoveActionUp.Click
        Dim methodsList As New ArrayList
        Dim checksList As New ArrayList
        Dim idxSelected As Integer

        With Me.clbPluginActions
            ' Is this item not at the top already?
            If .SelectedIndex <> 0 Then
                ' Loops through all plugin actions items
                For I As Integer = 0 To .Items.Count - 1
                    ' Move the selected item one position up
                    If I + 1 = .SelectedIndex Then
                        methodsList.Add(.Items.Item(.SelectedIndex))
                        checksList.Add(.GetItemCheckState(.SelectedIndex))

                        methodsList.Add(.Items.Item(I))
                        checksList.Add(.GetItemCheckState(I))

                        idxSelected = I
                        I = I + 1
                    Else
                        methodsList.Add(.Items.Item(I))
                        checksList.Add(.GetItemCheckState(I))
                    End If
                Next

                ' Clear all items in the plugin actions ListBox
                .Items.Clear()

                ' Fill the plugin actions ListBox with the new list
                For I As Integer = 0 To methodsList.Count - 1
                    .Items.Add(methodsList(I), checksList(I))
                Next

                ' Select the moved item
                .SetSelected(idxSelected, True)

                ' Also move the item in the Plugins Strcuture
                For Each pmItem As StructAvailablePlugins In Plugins
                    If pmItem.Name = Me.clbPluginsList.SelectedItem Then
                        For I As Integer = 0 To pmItem.Methods.Count - 1
                            pmItem.Methods(I) = methodsList(I)
                            pmItem.Checked(I) = checksList(I)
                        Next
                        Exit For
                    End If
                Next
            End If
        End With
    End Sub

    ' Moves the selected plugin action item one position down
    Private Sub btnMoveActionDown_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnMoveActionDown.Click
        Dim methodsList As New ArrayList
        Dim checksList As New ArrayList
        Dim idxSelected As Integer

        With Me.clbPluginActions
            ' Is this item not at the bottom already?
            If .SelectedIndex <> .Items.Count - 1 Then
                ' Loops through all plugin actions items
                For I As Integer = 0 To .Items.Count - 1
                    ' Move the selected item one position down
                    If I = .SelectedIndex Then
                        I = I + 1
                        idxSelected = I

                        methodsList.Add(.Items.Item(I))
                        checksList.Add(.GetItemCheckState(I))

                        methodsList.Add(.Items.Item(.SelectedIndex))
                        checksList.Add(.GetItemCheckState(.SelectedIndex))
                    Else
                        methodsList.Add(.Items.Item(I))
                        checksList.Add(.GetItemCheckState(I))
                    End If
                Next

                ' Clear all items in the plugin actions ListBox
                .Items.Clear()

                ' Fill the plugin actions ListBox with the new list
                For I As Integer = 0 To methodsList.Count - 1
                    .Items.Add(methodsList(I), checksList(I))
                Next

                ' Select the moved item
                .SetSelected(idxSelected, True)

                ' Also move the item in the Plugins Strcuture
                For Each pmItem As StructAvailablePlugins In Plugins
                    If pmItem.Name = Me.clbPluginsList.SelectedItem Then
                        pmItem.Methods = methodsList
                        pmItem.Checked = checksList

                        Exit For
                    End If
                Next
            End If
        End With
    End Sub

    ' Call methods to update check boxes
    Private Sub cbxNIC1_SelectedIndexChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cbxNIC1.SelectedIndexChanged
        UpdateComboBox(Me.cbxNIC1, Me.cbxNIC2, 0)

        ' Allow to enable/disable connection
        Me.chkEnableN1.Enabled = True
    End Sub

    Private Sub cbxNIC2_SelectedIndexChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cbxNIC2.SelectedIndexChanged
        UpdateComboBox(Me.cbxNIC2, Me.cbxNIC1, 1)

        ' Allow to enable/disable connection
        Me.chkEnableN2.Enabled = True
    End Sub

    ' Clear NIC2 combobox if NIC1 isn't configured
    Private Sub cbxNIC2_SelectionChangeCommitted(ByVal sender As Object, ByVal e As System.EventArgs) Handles cbxNIC2.SelectionChangeCommitted
        If Me.cbxNIC1.SelectedItem Is Nothing Then
            Me.cbxNIC2.SelectedIndex = -1
            Me.cbxNIC2.SelectedIndex = -1

            MessageBox.Show("You can't configure this connection without configuring the one on the left first.", "Unconfigurable", MessageBoxButtons.OK, MessageBoxIcon.Information)
        End If
    End Sub

#End Region

#Region " Form Events "

    ' Clear new/edit profile vars
    Private Sub frmNewProfile_FormClosing(ByVal sender As Object, ByVal e As System.Windows.Forms.FormClosingEventArgs) Handles Me.FormClosing
        newProfile = False
        editProfile = False
        profileName = Nothing
    End Sub

    ' Sets form initial display settings
    Private Sub frmNewProfile_Load(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.Load
        ' Do we have any plugins available?
        If Not AvailablePlugins Is Nothing Then
            ' Loops through all plugins available in the plugins structure
            For Each pmItem As StructAvailablePlugins In Plugins
                ' Add plugin name to the plugins list
                Me.clbPluginsList.Items.Add(pmItem.Name)

                ' Set this plugin methods to unchecked
                For I As Integer = 0 To pmItem.Checked.Count - 1
                    pmItem.Checked(I) = CheckState.Unchecked
                Next
            Next
        End If

        ' Fill initial values for both comboboxes
        Me.cbxNIC1.Items.AddRange(ShellItems.NicItems)
        Me.cbxNIC2.Items.AddRange(ShellItems.NicItems)

        ' Set label with the new profile name (or the one we are editing)
        Me.lblProfileName.Text = profileName

        ' If we are editing a profile, fill the form with it's settings
        If editProfile Then
            Dim NIC1 As StructConnSettings, NIC2 As StructConnSettings

            ' Initialize XML profiles file
            XML = New XMLParser(Application.StartupPath, "profiles", "Profiles")

            ' Gets NIC1 and NIC2 profile settings
            NIC1 = GetStructConnSettings(XML, profileName, "NIC1")
            NIC2 = GetStructConnSettings(XML, profileName, "NIC2")

            ' Sets in the global plugins ArrayList the correct check states
            Dim assignedPlugins As ArrayList
            assignedPlugins = SetPluginsMethodsCheckState(profileName)

            ' Sets this profile NIC1 settings
            If Not NIC1.Name = Nothing Then
                Me.cbxNIC1.SelectedItem = NIC1.Name
                Me.chkEnableN1.Checked = NIC1.Enable

                ' Enables/Disables controls related to NIC1 DHCP
                If NIC1.DHCP Then
                    Me.chkDHCPN1.Checked = True
                    ChangeDHCPFields(1)
                ElseIf NIC1.Enable Then
                    ' Fills all IPs for NIC1
                    Me.txtIPN1.Text = NIC1.IP.ToString
                    Me.txtSubnetN1.Text = NIC1.Subnet.ToString
                    If NIC1.Gateway IsNot Nothing Then Me.txtGatewayN1.Text = NIC1.Gateway.ToString
                    If NIC1.DNS1 IsNot Nothing Then Me.txtDNS1N1.Text = NIC1.DNS1.ToString
                    If NIC1.DNS2 IsNot Nothing Then Me.txtDNS2N1.Text = NIC1.DNS2.ToString
                End If

                ' Sets this profile NIC2 settings
                If Not NIC2.Name = Nothing Then
                    Me.cbxNIC2.SelectedItem = NIC2.Name
                    Me.chkEnableN2.Checked = NIC2.Enable

                    ' Enables/Disables controls related to NIC2 DHCP
                    If NIC2.DHCP Then
                        Me.chkDHCPN2.Checked = True
                        ChangeDHCPFields(2)
                    ElseIf NIC2.Enable Then
                        ' Fills all IPs for NIC2
                        Me.txtIPN2.Text = NIC2.IP.ToString
                        Me.txtSubnetN2.Text = NIC2.Subnet.ToString
                        If NIC2.Gateway IsNot Nothing Then Me.txtGatewayN2.Text = NIC2.Gateway.ToString
                        If NIC2.DNS1 IsNot Nothing Then Me.txtDNS1N2.Text = NIC2.DNS1.ToString
                        If NIC2.DNS2 IsNot Nothing Then Me.txtDNS2N2.Text = NIC2.DNS2.ToString
                    End If
                End If

                ' Set the assigned plugins to check state in the plugins list
                If assignedPlugins.Count > 0 Then
                    ' Loops through all plugins listed in the plugins list
                    For I As Integer = 0 To Me.clbPluginsList.Items.Count - 1
                        ' Loops through all the assigned plugins
                        For Each aPlugin As String In assignedPlugins
                            ' Set the state to check if we find a matching plugin
                            If Me.clbPluginsList.Items.Item(I) = aPlugin Then
                                Me.clbPluginsList.SetItemCheckState(I, CheckState.Checked)

                                Exit For
                            End If
                        Next
                    Next
                End If
            End If
        End If

        ' Do not allow to enable connections if no connection is selected
        If Me.cbxNIC1.SelectedItem Is Nothing Then
            Me.chkEnableN1.Enabled = False
        End If

        If Me.cbxNIC2.SelectedItem Is Nothing Then
            Me.chkEnableN2.Enabled = False
        End If
    End Sub

#End Region

End Class
