' This file is part of Network Switcher.

' Network Switcher is free software: you can redistribute it and/or modify
' it under the terms of the GNU General Public License as published by
' the Free Software Foundation, either version 3 of the License, or
' (at your option) any later version.

' Network Switcher is distributed in the hope that it will be useful,
' but WITHOUT ANY WARRANTY; without even the implied warranty of
' MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
' GNU General Public License for more details.

' You should have received a copy of the GNU General Public License
' along with Network Switcher. If not, see <http://www.gnu.org/licenses/>.
'
' Copyright 2007, Ricardo Amaral

Imports System
Imports System.Management
Imports Microsoft.Win32

Module NICMethods

#Region " Friend Methods "

    ' Method: Test if the Network Connections folder exists
    Friend Function GetNetworkConnectionsFolder(ByVal nFolder As String) As Shell32.Folder
        Try
            Dim sh As New Shell32.Shell()
            Dim cp As Shell32.Folder = sh.NameSpace(3) ' 3 = Control Panel
            Dim cpItems As Shell32.FolderItems = cp.Items()

            ' Go through all control panel items
            For Each item As Shell32.FolderItem In cpItems
                ' Found what we want?
                If item.Name = nFolder Then
                    Return CType(item.GetFolder, Shell32.Folder)
                End If
            Next
        Catch ex As Exception
            MessageBox.Show("Getting Network Connections Folder Error: " + ControlChars.CrLf + ex.Message, "Application Error", MessageBoxButtons.OK, MessageBoxIcon.Error)
        End Try

        Return Nothing
    End Function

    ' Method: Gets and returns the NIC folder item
    Friend Function GetNicFolderItem(ByVal nicName As String) As Shell32.FolderItem2
        Try
            Dim nicItems As Shell32.FolderItems = ShellItems.NicFolder.Items()

            ' Loops through all folder items
            For Each item As Shell32.FolderItem In nicItems
                If item.Name = nicName Then
                    Return CType(item, Shell32.FolderItem2)
                End If
            Next
        Catch ex As Exception
            MessageBox.Show("Getting NIC Folder Error: " + ControlChars.CrLf + ex.Message, "Application Error", MessageBoxButtons.OK, MessageBoxIcon.Error)
        End Try

        Return Nothing
    End Function

    ' Method: Get all network connections cards from Network Connections folder
    Friend Function GetAllNetworkAdapters() As String()
        Try
            Dim nicItems As Shell32.FolderItems = ShellItems.NicFolder.Items()
            Dim alNicItems As New ArrayList

            ' Loops through all folder items
            For Each item As Shell32.FolderItem In nicItems
                alNicItems.Add(item.Name)
            Next

            Dim aNicItems(alNicItems.Count - 1) As String
            Dim I As Integer

            ' Create a String array from an ArrayList
            For Each item As String In alNicItems
                aNicItems(I) = item
                I += 1
            Next

            Return aNicItems
        Catch ex As Exception
            MessageBox.Show("Getting All Network Adapters Error: " + ControlChars.CrLf + ex.Message, "Application Error", MessageBoxButtons.OK, MessageBoxIcon.Error)
        End Try

        Return Nothing
    End Function

    ' Method: Enables or disables a network connection
    Friend Sub ToggleNic(ByVal nicName As String, ByVal nicMode As ToggleMode)
        Try
            Dim verb As Shell32.FolderItemVerb = Nothing
            Dim networkConnection As Shell32.FolderItem2 = GetNicFolderItem(nicName)

            ' Get network connection status
            Dim nicStatus As Boolean '= CBool(IsNicEnabled(networkConnection, verb))

            ' Is the connection enabled?
            If nicStatus Then
                ' Disable the connection?
                If nicMode = ToggleMode.Disable Then
                    verb.DoIt()
                    System.Threading.Thread.Sleep(1000)
                End If
            Else
                ' Enable the connection?
                If nicMode = ToggleMode.Enable Then
                    verb.DoIt()
                    System.Threading.Thread.Sleep(2000)
                End If
            End If
        Catch ex As Exception
            MessageBox.Show("Toggle Network Adapter Error: " + ControlChars.CrLf + ex.Message, "Application Error", MessageBoxButtons.OK, MessageBoxIcon.Error)
        End Try
    End Sub

    ' Method: Enables DHCP using WMI in this network adapter
    Friend Function WMIEnableDHCP(ByVal nicName As String) As Boolean
        Try
            ' Get this network adapter information
            Dim mos As ManagementObjectSearcher
            mos = New ManagementObjectSearcher("SELECT * " + _
                                               "FROM Win32_NetworkAdapter " + _
                                               "WHERE NetConnectionID = '" + nicName + "'")

            For Each moAdapter As ManagementObject In mos.Get
                ' Get more information for this network adpater
                mos = New ManagementObjectSearcher("ASSOCIATORS OF {Win32_NetworkAdapter.DeviceID='" + _
                                                   moAdapter("DeviceID").ToString + "'} " + _
                                                   "WHERE AssocClass=Win32_NetworkAdapterSetting")

                ' Enable DHCP for this network adapater (only looping once)
                For Each moAdapterAssoc As ManagementObject In mos.Get()
                    Dim newDNS As ManagementBaseObject = moAdapterAssoc.GetMethodParameters("SetDNSServerSearchOrder")
                    newDNS("DNSServerSearchOrder") = Nothing
                    Dim errorCode As Object = moAdapterAssoc.InvokeMethod("EnableDHCP", New Object() {})
                    moAdapterAssoc.InvokeMethod("SetDNSServerSearchOrder", newDNS, Nothing)

                    ' Return false or true depending on the error code
                    Return CBool(errorCode)
                Next
            Next
        Catch ex As Exception
            MessageBox.Show("Enabling DHCP Error (WMI): " + ControlChars.CrLf + ex.Message, "Application Error", MessageBoxButtons.OK, MessageBoxIcon.Error)
        End Try

        ' We have an error if we are here
        Return True
    End Function

    ' Method: Enables static IP's using WMI
    Friend Sub WMIEnableStatic(ByVal nicName As String, ByVal IpAddresses As String, ByVal SubnetMask As String, ByVal Gateway As String, ByVal DnsSearchOrder As String)
        Try
            ' Get this network adapter information
            Dim mos As ManagementObjectSearcher
            mos = New ManagementObjectSearcher("SELECT * " + _
                                               "FROM Win32_NetworkAdapter " + _
                                               "WHERE NetConnectionID = '" + nicName + "'")

            For Each moAdapter As ManagementObject In mos.Get
                ' Get more information for this network adpater
                mos = New ManagementObjectSearcher("ASSOCIATORS OF {Win32_NetworkAdapter.DeviceID='" + _
                                                   moAdapter("DeviceID").ToString + "'} " + _
                                                   "WHERE AssocClass=Win32_NetworkAdapterSetting")

                ' Enable static IP's for this network adapater (only looping once)
                For Each moAdapterAssoc As ManagementObject In mos.Get()
                    ' Define new gateway
                    Dim newGate As ManagementBaseObject = moAdapterAssoc.GetMethodParameters("SetGateways")
                    newGate("DefaultIPGateway") = New String() {Gateway}
                    newGate("GatewayCostMetric") = New Integer() {1}

                    ' Define new IP and subnet mask
                    Dim newIP As ManagementBaseObject = moAdapterAssoc.GetMethodParameters("EnableStatic")
                    newIP("IPAddress") = New String() {IpAddresses}
                    newIP("SubnetMask") = New String() {SubnetMask}

                    ' Define new DNS
                    Dim newDNS As ManagementBaseObject = moAdapterAssoc.GetMethodParameters("SetDNSServerSearchOrder")
                    newDNS("DNSServerSearchOrder") = DnsSearchOrder.Split(",")

                    ' Invoke all methods to fully enable static IPs
                    moAdapterAssoc.InvokeMethod("EnableStatic", newIP, Nothing)
                    moAdapterAssoc.InvokeMethod("SetGateways", newGate, Nothing)
                    moAdapterAssoc.InvokeMethod("SetDNSServerSearchOrder", newDNS, Nothing)
                Next
            Next
        Catch ex As Exception
            MessageBox.Show("Enabling Static IP's Error (WMI): " + ControlChars.CrLf + ex.Message, "Application Error", MessageBoxButtons.OK, MessageBoxIcon.Error)
        End Try
    End Sub

    ' Method: Enables DHCP using the Registry
    Friend Sub RegistryEnableDHCP(ByVal nicName As String)
        Try
            Dim regKey As RegistryKey

            ' Set registry key for opening
            regKey = Registry.LocalMachine.OpenSubKey("SYSTEM\CurrentControlSet\Services\Tcpip\Parameters\Interfaces\" + _
            WMIGetNetworkAdapterSettingID(nicName), True)

            ' Clear IP Address, Subnet Mask, Gateway, Name Servers and enable DHCP
            regKey.SetValue("IPAddress", "")
            regKey.SetValue("SubnetMask", "")
            regKey.SetValue("DefaultGateway", "")
            regKey.SetValue("DefaultGatewayMetric", "")
            regKey.SetValue("NameServer", "")
            regKey.SetValue("EnableDHCP", 1)

            ' Close opened registry key
            regKey.Close()
        Catch ex As Exception
            MessageBox.Show("Enabling DHCP Error (Registry): " + ControlChars.CrLf + ex.Message, "Application Error", MessageBoxButtons.OK, MessageBoxIcon.Error)
        End Try
    End Sub

    ' Method: Checks if the NIC is enabled or not
    'Friend Function IsNICActive(ByVal nicName As String) As Boolean
    '    Dim mos As ManagementObjectSearcher
    '    mos = New ManagementObjectSearcher("SELECT * FROM Win32_NetworkAdapter")

    '    ' Loops through all NIC adapters
    '    For Each moAdapter As ManagementObject In mos.Get
    '        ' Can we configure this NIC TCP/IP protocol?
    '        If moAdapter("NetConnectionID") IsNot Nothing Then
    '            ' Is this the NIC? It's enabled then...
    '            If moAdapter("Caption").Equals(WMIGetNICCaptionOrSettingID(nicName, False)) Then
    '                Return True
    '            End If
    '        End If
    '    Next

    '    ' The NIC was not found (not enabled yet)
    '    Return False
    'End Function

#End Region

#Region " Private Methods "

    ' Method: Checks if a certain network connection is enabled
    'Private Function IsNicEnabled(ByVal networkConnection As Shell32.FolderItem2, ByRef enableDisableVerb As Shell32.FolderItemVerb)
    '    Dim verbs As Shell32.FolderItemVerbs = networkConnection.Verbs

    '    ' Go through all item verbs
    '    For Each verb As Shell32.FolderItemVerb In verbs
    '        ' Do we have a enable or disable verb?
    '        If verb.Name = Settings.EnableVerb Then
    '            enableDisableVerb = verb
    '            Return False
    '        ElseIf verb.Name = Settings.DisableVerb Then
    '            enableDisableVerb = verb
    '            Return True
    '        End If
    '    Next

    '    ' No verb was found
    '    Return Nothing
    'End Function

    ' Method: Gets the NIC Caption
    Friend Function WMIGetNetworkAdapterSettingID(ByVal nicName As String) As String
        Try
            ' Get this network adapter information
            Dim mos As ManagementObjectSearcher
            mos = New ManagementObjectSearcher("SELECT * " + _
                                               "FROM Win32_NetworkAdapter " + _
                                               "WHERE NetConnectionID = '" + nicName + "'")

            For Each moAdapter As ManagementObject In mos.Get
                ' Get more information for this network adpater
                mos = New ManagementObjectSearcher("ASSOCIATORS OF {Win32_NetworkAdapter.DeviceID='" + _
                                                   moAdapter("DeviceID").ToString + "'} " + _
                                                   "WHERE AssocClass=Win32_NetworkAdapterSetting")

                ' Returns this adapter setting ID (only looping once)
                For Each moAdapterAssoc As ManagementObject In mos.Get()
                    Return moAdapterAssoc("SettingID")
                Next
            Next
        Catch ex As Exception
            MessageBox.Show("Getting SettingID Error (WMI): " + ControlChars.CrLf + ex.Message, "Application Error", MessageBoxButtons.OK, MessageBoxIcon.Error)
        End Try

        Return Nothing
    End Function

#End Region

End Module
